#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef JFB_WITH_LIBHANGUL
#include <stdio.h>
#include <iconv.h>
#include <endian.h>
#include <string.h>
#include "conv.h"

int
conv_str(const char *fromenc, const char *toenc, unsigned char *fromstr, unsigned char *tostr)
{
	int inbytesleft, outbytesleft, convbytes;
	char *inbuf, *outbuf;

	size_t ret = (size_t)-1;
	iconv_t cd = (iconv_t)-1;

	inbuf = fromstr;
	outbuf = tostr;

	cd = iconv_open(toenc, fromenc);
	if(cd == (iconv_t)-1) {
		return -1;
	}

	if(!strncmp(fromenc, "UCS-4", 5)) {
		int size = 0;
#if __BYTE_ORDER == __BIG_ENDIAN
		/* FIXME: not tested */
		int i, temp;
		while(fromstr[size * 4 + 3] != 0) {
			if(size > CONV_SIZE * 4)
				break;
			size++;
		}
		inbytesleft = size * 4;
#elif __BYTE_ORDER == __LITTLE_ENDIAN
		int i;
		unsigned char temp;
		while(fromstr[size * 4] != 0 ||
		      fromstr[size * 4 + 1] != 0 ||
		      fromstr[size * 4 + 2] != 0 ||
		      fromstr[size * 4 + 3] != 0) {
			if(size > CONV_SIZE * 4)
				break;
			size++;
		}
		inbytesleft = size * 4;

		/* swap bytes */
		for(i = 0; i < size; i++) {
			temp = fromstr[i * 4];
			fromstr[i * 4] = fromstr[i * 4 + 3];
			fromstr[i * 4 + 3] = temp;

			temp = fromstr[i * 4 + 1];
			fromstr[i * 4 + 1] = fromstr[i * 4 + 2];
			fromstr[i * 4 + 2] = temp;
		}
#else
#error "FIXME: endian type undefined"
#endif
	}
	else {
		inbytesleft = strlen(fromstr);
	}
	outbytesleft = CONV_SIZE;
	ret = iconv(cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft);

	if(ret == (size_t)-1) {
		iconv_close(cd);
		return -1;
	}
	iconv_close(cd);

	convbytes = CONV_SIZE - outbytesleft;

#if __BYTE_ORDER == __LITTLE_ENDIAN
	if(!strncmp(toenc, "UCS-4", 5)) {
		int i;
		unsigned char temp;
		/* swap bytes */
		for(i = 0; i < convbytes; i++) {
			temp = tostr[i * 4];
			tostr[i * 4] = tostr[i * 4 + 3];
			tostr[i * 4 + 3] = temp;

			temp = tostr[i * 4 + 1];
			tostr[i * 4 + 1] = tostr[i * 4 + 2];
			tostr[i * 4 + 2] = temp;
		}
		convbytes = convbytes / 4;
	}
#endif
	return convbytes;
}

#if 0
int
main(void)
{
	unsigned int ucs4str[CONV_SIZE] = {0, };
	unsigned char utf8str[] = "한글rㄱ";
	unsigned char utf8str2[CONV_SIZE] = {0, };
	int ret;
	fprintf(stderr, "ORIG: %s\r\n", utf8str);
	//ret = CONV_euckr_to_utf8(utf8str, utf8str2);
	ret = CONV_utf8_to_ucs4(utf8str, (char *)ucs4str);
	fprintf(stderr, "[%d]%s\r\n", ret, utf8str);
	int i = 0;
	for(i = 0; i < CONV_SIZE; i++) {
		fprintf(stderr, "[%d] ", ucs4str[i]);
	}
	ret = CONV_ucs4_to_utf8((char *)ucs4str, utf8str2);
	fprintf(stderr, "[%d]%s\r\n", ret, utf8str2);
	fprintf(stderr, "END\r\n");
	return 0;
}
#endif
#endif
