#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef JFB_WITH_LIBHANGUL
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <locale.h>
#include <langinfo.h>
#include <sys/ioctl.h>
#include <hangul.h>
#include "conv.h"
#include "hinput.h"

HangulInputContext *hic = NULL;

static int hkeyboard;
int hinput_encoding;

/* internal use only */
int get_shift_state(void)
{
	int na = 6, ns = 0;
	if(ioctl(fileno(stdin), TIOCLINUX, &na) == 0) ns = na;
	return ns;
}

int hinput_init(char *keyboard_type, char *input_encoding)
{
	if(!keyboard_type || strcmp(keyboard_type, "2") == 0)
		hkeyboard = HANGUL_KEYBOARD_2;
	else if(strcmp(keyboard_type, "32") == 0)
		hkeyboard = HANGUL_KEYBOARD_32;
	else if(strcmp(keyboard_type, "3final") == 0 ||
		strcmp(keyboard_type, "3") == 0)
		hkeyboard = HANGUL_KEYBOARD_3FINAL;
	else if(strcmp(keyboard_type, "390") == 0)
		hkeyboard = HANGUL_KEYBOARD_390;
	else if(strcmp(keyboard_type, "3noshift") == 0)
		hkeyboard = HANGUL_KEYBOARD_3NOSHIFT;
	else if(strcmp(keyboard_type, "3yetgul") == 0)
		hkeyboard = HANGUL_KEYBOARD_3YETGUL;
	else {
		hkeyboard = HANGUL_KEYBOARD_NONE;
		hinput_encoding = HINPUT_ENCODING_NONE;
		return 1;
	}
	
	if(!input_encoding || strcmp(input_encoding, "locale") == 0) {
		setlocale(LC_ALL, "");
		input_encoding = nl_langinfo(CODESET);
	}
	if(strcmp(input_encoding, "UTF-8") == 0 ||
	   strcmp(input_encoding, "utf-8") == 0 ||
	   strcmp(input_encoding, "UTF8") == 0 ||
	   strcmp(input_encoding, "utf8") == 0) {
		hic = hangul_ic_new(hkeyboard);
		if(hic == NULL) {
			hinput_encoding = HINPUT_ENCODING_NONE;
			return 0;
		}
		hinput_encoding = HINPUT_ENCODING_UTF8;
	}
	else if(strcmp(input_encoding, "EUC-KR") == 0 ||
		strcmp(input_encoding, "euc-kr") == 0 ||
		strcmp(input_encoding, "EUCKR") == 0 ||
		strcmp(input_encoding, "euckr") == 0 ||
		strcmp(input_encoding, "eucKR") == 0) {
		hic = hangul_ic_new(hkeyboard);
		if(hic == NULL) {
			hinput_encoding = HINPUT_ENCODING_NONE;
			return 0;
		}
		hinput_encoding = HINPUT_ENCODING_EUCKR;
	}
	else
		hinput_encoding = HINPUT_ENCODING_NONE;
		
	return 1;
}

void hinput_exit(void)
{
	if(hic != NULL)
		hangul_ic_delete(hic);
}

int hinput_try_commit(TTerm *p, u_int *commit_flag)
{
	u_char commit[CONV_SIZE] = {0, };
	u_char commit_temp[CONV_SIZE] = {0, };
	ucschar *commit_ucs4;
	u_int ret;

	commit_ucs4 = hangul_ic_get_commit_string(hic);
	if(hinput_encoding == HINPUT_ENCODING_UTF8) {
		ret = CONV_ucs4_to_utf8((char *)commit_ucs4, commit);
		if(ret == -1) // iconv error
			return 0;
	}
	else if(hinput_encoding == HINPUT_ENCODING_EUCKR) {
		/* iconv from ucs4 to euc-kr doesn't work. why? */
		ret = CONV_ucs4_to_utf8((char *)commit_ucs4, commit_temp);
		if(ret == -1) // iconv error
			return 0;
		ret = CONV_utf8_to_euckr(commit_temp, commit);
		if(ret == -1)
			return 0;
	}
	else { // unknown encoding
		return 0;
	}

	if(strlen(commit) > 0) {
		write(p->ptyfd, commit, strlen(commit));
		*commit_flag = 1;
	}
	return 1;
}

int hinput_try_preedit(TTerm *p, u_int state, u_int *commit_flag)
{
	u_char preedit[CONV_SIZE] = {0, };
	u_char preedit_temp[CONV_SIZE] = {0, };
	static u_char preedit_save[CONV_SIZE];
	ucschar *preedit_ucs4;
	u_int ret;

	if(state == 1 && *commit_flag == 1) {
		if(strlen(preedit_save) > 0) {
			int i;
			tvterm_emulate(&(p->vterm), preedit_save, strlen(preedit_save));
			p->vterm.pen.x -= 2;
			tvterm_refresh(&(p->vterm));
			for(i = 0; i < CONV_SIZE; i++) {
				preedit_save[i] = 0;
			}
		}
		*commit_flag = 0;
		return 1;
	}

	preedit_ucs4 = hangul_ic_get_preedit_string(hic);
	if(hinput_encoding == HINPUT_ENCODING_UTF8) {
		ret = CONV_ucs4_to_utf8((char *)preedit_ucs4, preedit);
		if(ret == -1) // iconv error
			return 0;
	}
	else if(hinput_encoding == HINPUT_ENCODING_EUCKR) {
		/* iconv from ucs4 to euc-kr doesn't work. why? */
		ret = CONV_ucs4_to_utf8((char *)preedit_ucs4, preedit_temp);
		if(ret == -1) // iconv_error
			return 0;
		ret = CONV_utf8_to_euckr(preedit_temp, preedit);
		if(ret == -1)
			return 0;
	}
	else { // unknown encoding
		return 0;
	}

	if(strlen(preedit) >= 0 && state == 0 && *commit_flag == 0) {
		if(strlen(preedit) == 0) {
			preedit[0] = ' ';
			preedit[1] = ' ';
			preedit[2] = 0;
		}
		tvterm_emulate(&(p->vterm), preedit, strlen(preedit));
		p->vterm.pen.x -= 2;
		tvterm_refresh(&(p->vterm));
	}
	else if(strlen(preedit) > 0 && state == 0 && *commit_flag == 1) {
		// save preedit char
		strncpy(preedit_save, preedit, CONV_SIZE);
	}
	return 1;
}

int hinput_process(TTerm *p, u_char *buf, int ret, u_int *preedit_after_commit)
{
	static u_int hstate = 0;
	u_int i, c, fret;
	if(buf[0] == KEY_ESC && hstate) {
		hangul_ic_flush(hic);
		hinput_try_commit(p, preedit_after_commit);
		hstate = 0;
	}
	if(ret == 1 && buf[0] == KEY_SPACE && get_shift_state()) {
		if(hstate) {
			hangul_ic_flush(hic);
			hinput_try_commit(p, preedit_after_commit);
			hstate = 0;
		}
		else
			hstate = 1;
	}
	else if(hstate) {
		for(i = 0; i < ret; i++) {
			c = buf[i];
			if(c == KEY_BACKSPACE) {
				if(!hangul_ic_backspace(hic)) {
					write(p->ptyfd, &c, 1);
					*preedit_after_commit = 1;
				}
				continue;
			}
			fret = hangul_ic_filter(hic, c);
			if(!fret) {
				hinput_try_commit(p, preedit_after_commit);
				write(p->ptyfd, &c, 1);
				*preedit_after_commit = 1;
				continue;
			}
			hinput_try_commit(p, preedit_after_commit);
		}
		hinput_try_preedit(p, 0, preedit_after_commit);
	}
	else
		return 0;
	return 1;
}
#endif
